import sys
import numpy as np
import random
import argparse

from .handlers import CHARGE
from . import constants

def parse_args():
    parser = argparse.ArgumentParser(
        prog="structgen",
        description = (
            "Generate initial structures in LAMMPS format for glassy systems "
            "given density, atom masses and counts."
        ),
        formatter_class = argparse.RawTextHelpFormatter,
    )

    subparsers = parser.add_subparsers(dest = "command", required = True, metavar = "")

    charge = subparsers.add_parser("charge", help = "Argument parser for generating initial glass structures in lammps format compliant with atom_style CHARGE.")

    charge.add_argument(
        "-a", "--atom",
        metavar = "",
        nargs = "+",
        action = "append",
        required = True,
        help = (
            "Provide <mass> [<count> <charge>] for an atom type.\n"
            "Defaults: count=1, charge=0.0.\n"
            "Repeat for multiple atom types: -a 28.085 -a 16.00 2 -1.2"
        ),
    )


    charge.add_argument(
        "-d", "--density",
        type = float,
        required = True,
        metavar = "",
        help = "Desired density of the output structure in g/cm^3.",
    )

    charge.add_argument(
        "-b", "--buffer",
        type=float,
        default = constants.DEFAULT_BUFFER_PERCENT,
        metavar = "",
        help = "Spacing between simulation region walls in %% of box size (default: 5).",
    )

    charge.add_argument(
        "-f", "--factor",
        type = int,
        default = constants.DEFAULT_ATOM_FACTOR,
        metavar = "",
        help = "Scale factor applied to all atom counts (default: 1).",
    )

    charge.add_argument(
        "-o", "--output",
        default = constants.DEFAULT_FILENAME,
        metavar = "",
        help = "Name of the resulting structure file (default: initial.structure).",
    )

    charge.set_defaults(handler_class = CHARGE)

    return parser.parse_args()

def gen_header(handler):

    header = str()
    header += "#Generated by structgen utility git@github.com:superde1fin/structgen.git\n\n"
    header += f"{handler.get_natoms()} atoms\n"
    header += f"{handler.get_natom_types()} atom types\n"

    sides = handler.get_sim_region_sides()
    header += f"\n0 {sides[0]} xlo xhi"
    header += f"\n0 {sides[1]} ylo yhi"
    header += f"\n0 {sides[2]} zlo zhi"

    header += "\n\n"
    return header


def gen_atoms(handler):

    atoms = str()
    column_headers = ' '.join(handler.get_atom_attrs())
    atoms += f"Atoms # {handler.get_atom_style()} | {column_headers}\n\n"

    atom_data = handler.get_atom_data()

    #Add atom lines and convert necesary elements to integers
    indices_to_int = [0, 1]
    atoms += "\n".join(" ".join(str(int(x)) if i in indices_to_int else str(x) for i, x in enumerate(row)) for row in atom_data)

    return atoms


def main():
    args = parse_args()

    handler = args.handler_class(args)

    file_str = str()

    file_str += gen_header(handler)
    file_str += gen_atoms(handler)

    with open(handler.get_outfile_name(), "w") as file:
        file.write(file_str)

        

if __name__ == "__main__":
    main()
